@@ -1,17 +1,20 @@
 package WebAPI::DBIC::Resource::Base;
-$WebAPI::DBIC::Resource::Base::VERSION = '0.001010'; # TRIAL
+$WebAPI::DBIC::Resource::Base::VERSION = '0.003001'; # TRIAL
 
 use Moo;
 extends 'Web::Machine::Resource';
 
+require WebAPI::HTTP::Throwable::Factory;
+
 
 has http_auth_type => (
    is => 'ro',
+   default => 'Basic',
 );
 
 has throwable => (
     is => 'rw',
-    required => 1,
+    default => 'WebAPI::HTTP::Throwable::Factory',
 );
 
 
@@ -29,7 +32,7 @@ WebAPI::DBIC::Resource::Base
 
 =head1 VERSION
 
-version 0.001010
+version 0.003001
 
 =head1 DESCRIPTION
 
