@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::GenericItem;
-$WebAPI::DBIC::Resource::GenericItem::VERSION = '0.001010'; # TRIAL
+$WebAPI::DBIC::Resource::GenericItem::VERSION = '0.003001'; # TRIAL
 
 use Moo;
 use namespace::clean;
@@ -8,9 +8,13 @@ extends 'WebAPI::DBIC::Resource::GenericCore';
 with    'WebAPI::DBIC::Resource::Role::Item',
         'WebAPI::DBIC::Resource::Role::ItemWritable',
         # Enable HAL support:
-        'WebAPI::DBIC::Resource::Role::DBIC_HAL',
-        'WebAPI::DBIC::Resource::Role::ItemHAL',
-        'WebAPI::DBIC::Resource::Role::ItemWritableHAL',
+        'WebAPI::DBIC::Resource::HAL::Role::DBIC',
+        'WebAPI::DBIC::Resource::HAL::Role::Item',
+        'WebAPI::DBIC::Resource::HAL::Role::ItemWritable',
+        # Enable JSON API support:
+        'WebAPI::DBIC::Resource::JSONAPI::Role::DBIC',
+        'WebAPI::DBIC::Resource::JSONAPI::Role::Item',
+        'WebAPI::DBIC::Resource::JSONAPI::Role::ItemWritable',
         ;
 
 1;
@@ -27,7 +31,7 @@ WebAPI::DBIC::Resource::GenericItem
 
 =head1 VERSION
 
-version 0.001010
+version 0.003001
 
 =head1 NAME
 
