@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::Role::DBIC;
-$WebAPI::DBIC::Resource::Role::DBIC::VERSION = '0.001010'; # TRIAL
+$WebAPI::DBIC::Resource::Role::DBIC::VERSION = '0.003001'; # TRIAL
 
 use Carp qw(croak confess);
 use Devel::Dwarn;
@@ -25,9 +25,13 @@ has writable => (
    is => 'ro',
 );
 
+has type_namer => (
+   is => 'ro',
+);
+
 has prefetch => (
     is => 'rw',
-    default => sub { {} },
+    default => sub { [] },
 );
 
 
@@ -73,7 +77,7 @@ sub web_machine_resource {
         request  => $self->request,
         response => $self->request->new_response,
         throwable => $self->throwable,
-        prefetch  => {}, # don't propagate prefetch by default
+        prefetch  => [], # don't propagate prefetch by default
         set => undef,
         id => undef,
         #  XXX others? which and why? generalize
@@ -154,7 +158,7 @@ WebAPI::DBIC::Resource::Role::DBIC
 
 =head1 VERSION
 
-version 0.001010
+version 0.003001
 
 =head1 NAME
 
