@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::Role::DBICParams;
-$WebAPI::DBIC::Resource::Role::DBICParams::VERSION = '0.001010'; # TRIAL
+$WebAPI::DBIC::Resource::Role::DBICParams::VERSION = '0.003001'; # TRIAL
 
 use Carp;
 use Scalar::Util qw(blessed);
@@ -30,7 +30,7 @@ sub malformed_request {
 # used to a) define order that params are handled,
 # and b) to force calling of a handler even if param is missing
 sub get_param_order {
-    return qw(page rows order);
+    return qw(page rows sort);
 }
 
 
@@ -57,7 +57,19 @@ sub handle_request_params {
         die "Explicit search_criteria param not allowed"
             if $param eq 'search_criteria';
 
-        $queue{$param} = $v[0];
+        # for parameters with names like foo[x]=3&foo[y]=4
+        # we accumulate the value as a hash { x => 3, y => 4 }
+        if ($param =~ /^(\w+)\[(\w+)\]$/) {
+            die "$param=$v[0] can't follow $param=$queue{$param} parameter\n"
+                if $queue{$1} and not ref $queue{$1};
+            $queue{$1}{$2} = $v[0];
+        }
+        else {
+            die "$param=$v[0] can't follow $param=$queue{$param} parameter\n"
+                if $queue{$param} and ref $queue{$param};
+            $param = 'sort' if $param eq 'order'; # XXX back-compat
+            $queue{$param} = $v[0];
+        }
     }
 
     # call handlers in desired order, then any remaining ones
@@ -107,58 +119,107 @@ sub _handle_search_criteria_param {
     return;
 }
 
-
 sub _handle_prefetch_param {
     my ($self, $value) = @_;
 
+    # Prefetchs/join in DBIC accepts either:
+    #   prefetch => relname OR
+    #   prefetch => [relname1, relname2] OR
+    #   prefetch => {relname1 => relname_on_relname1} OR
+    #   prefetch => [{relname1 => [{relname_on_relname1 => relname_on_relname_on_relname1}, other_relname_on_relaname1]},relname2] ETC
+
+    # Noramalise all prefetches to most complicated form.
     # eg &prefetch=foo,bar  or  &prefetch.json={...}
-    my $prefetch = (ref $value)
-        ? $value
-        : { map { $_ => {} } split(',', $value||"") };
+    my $prefetch = $self->_resolve_prefetch($value, $self->set->result_source);
 
-    return unless %$prefetch;
+    return unless scalar @$prefetch;
+    # XXX hack?: perhaps use {embedded}{$key} = sub { ... };
+    # see lib/WebAPI/DBIC/Resource/Role/DBIC.pm
+    $self->prefetch( $prefetch ); # include self, even if deleted below
+    $prefetch = [grep { !defined $_->{self}} @$prefetch];
 
-    my $result_class = $self->set->result_class;
+    my $prefetch_or_join = $self->param('fields') ? 'join' : 'prefetch';
+    $self->set( $self->set->search_rs(undef, { $prefetch_or_join => $prefetch }))
+        if scalar @$prefetch;
 
+    return;
+}
+
+sub _resolve_prefetch {
+    my ($self, $prefetch, $result_class) = @_;
     my @errors;
-    for my $prefetch (keys %$prefetch) {
 
-        next if $prefetch eq 'self'; # used in POST/PUT handling
+    # Here we recursively resolve each of the prefetches to normalise them all to the most complicated
+    # form that can exist. The results will be a ArrayRef of HashRefs that can be passed to DBIC
+    # directly.
+    # This code is largely taken from the _resolve_join subroutine in DBIx::Class
 
-        my $rel;
-        try {
-            $rel = $result_class->relationship_info($prefetch);
-            local $SIG{__DIE__}; # avoid strack trace from these dies:
-            die "no relationship with that name"
-                if not $rel;
-            die "relationship is $rel->{attrs}{accessor} but only single, filter and multi are supported\n"
-                if not $rel->{attrs}{accessor} =~ m/^(?:single|filter|multi)$/ # sanity
+    return [] unless defined $prefetch and length $prefetch;
+    my @return;
+
+    if (ref $prefetch eq 'ARRAY') {
+        push @return, map {
+            $self->_resolve_prefetch($_, $result_class)
+        } @$prefetch;
+    } elsif (ref $prefetch eq 'HASH') {
+        for my $rel (keys %$prefetch) {
+            next if $rel eq 'self';
+
+            if (my @validate_errors = $self->_validate_relationship($result_class, $rel)) {
+                push @errors, @validate_errors;
+            } else {
+                push @return, {
+                    $rel => $self->_resolve_prefetch($prefetch->{$rel}, $result_class->related_source($rel))
+                };
+            }
         }
-        catch {
-            push @errors, {
-                $prefetch => $_,
-                _meta => {
-                    relationship => $rel,
-                    relationships => [ $result_class->relationships ]
-                }, # XXX
-            };
+    } elsif (ref $prefetch) {
+        push @errors,
+            "No idea how to resolve prefetch reftype ".ref $prefetch;
+    } else {
+        for my $rel (split ',', $prefetch) {
+            my @validate_errors = $self->_validate_relationship($result_class, $rel);
+            if ($rel ne 'self' && scalar @validate_errors) {
+                push @errors, @validate_errors;
+            } else {
+                push @return, {
+                    $rel => {},
+                };
+            }
         }
     }
 
     $self->throwable->throw_bad_request(400, errors => \@errors)
         if @errors;
 
-    # XXX hack?: perhaps use {embedded}{$key} = sub { ... };
-    # see lib/WebAPI/DBIC/Resource/Role/DBIC.pm
-    $self->prefetch({ %$prefetch }); # include self, even if deleted below
-
-    delete $prefetch->{self};
-    $self->set( $self->set->search_rs(undef, { prefetch => $prefetch }))
-        if %$prefetch;
-
-    return;
+    return \@return;
 }
 
+sub _validate_relationship {
+    my ($self, $result_class, $rel) = @_;
+    my @errors;
+
+    my $rel_info;
+    try {
+        $rel_info = $result_class->relationship_info($rel);
+        local $SIG{__DIE__}; # avoid strack trace from these dies:
+        die "no relationship with that name\n"
+            if not $rel_info;
+        die "relationship is $rel_info->{attrs}{accessor} but only single, filter and multi are supported\n"
+            if not $rel_info->{attrs}{accessor} =~ m/^(?:single|filter|multi)$/; # sanity
+    }
+    catch {
+        push @errors, {
+            $rel => $_,
+            _meta => {
+                relationship => $rel_info,
+                relationships => [ sort $result_class->relationships ]
+            }, # XXX
+        };
+    };
+
+    return @errors;
+}
 
 sub _handle_fields_param {
     my ($self, $value) = @_;
@@ -178,14 +239,6 @@ sub _handle_fields_param {
             parameter => "invalid fields clause",
             _meta => { fields => $field, }, # XXX
         }]) if not defined $field;
-        # sadly columns=>[...] doesn't work to limit the fields of prefetch relations
-        # so we disallow that for now. It's possible we could achieve the same effect
-        # using explicit join's for non-has-many rels, or perhaps using
-        # as_subselect_rs
-        $self->throwable->throw_bad_request(400, errors => [{
-            parameter => "invalid fields clause - can't refer to prefetch relations at the moment",
-            _meta => { fields => $field, }, # XXX
-        }]) if $field =~ m/\./;
     }
 
     $self->set( $self->set->search_rs(undef, { columns => \@columns }) )
@@ -195,24 +248,44 @@ sub _handle_fields_param {
 }
 
 
-sub _handle_order_param {
+sub _handle_sort_param {
     my ($self, $value) = @_;
     my @order_spec;
 
+    # to support sort[typename]=... we need to be able to make type names
+    # to relationship names that map to the type and are included in the query
+    # (there might be more than one relationship on 'me' that leads to
+    # the same resource type so there's a potential ambiguity)
+    if (ref $value) {
+        $self->throwable->throw_bad_request(400, errors => [{
+            parameter => "per-type sort specifiers are not supported yet",
+            _meta => { sort => $value, }, # XXX
+        }]);
+    }
+
     if (not defined $value) {
         $value = (join ",", map { "me.$_" } $self->set->result_source->primary_columns);
     }
 
-    for my $clause (split /\s*,\s*/x, $value) {
+    for my $clause (split /,/, $value) {
+
         # we take care to avoid injection risks
-        my ($field, $dir) = ($clause =~ /^ ([a-z0-9_\.]*)\b (?:\s+(asc|desc))? \s* $/xi);
+        my ($field, $dir);
+        if ($clause =~ /^ ([a-z0-9_\.]*)\b (?:\s+(asc|desc))? $/xi) {
+            ($field, $dir) = ($1, $2 || 'asc');
+        }
+        elsif ($clause =~ /^ (-?) ([a-z0-9_\.]*)$/xi) {
+            ($field, $dir) = ($2, ($1) ? 'desc' : 'asc');
+        }
+
         unless (defined $field) {
             $self->throwable->throw_bad_request(400, errors => [{
                 parameter => "invalid order clause",
                 _meta => { order => $clause, }, # XXX
             }]);
         }
-        $dir ||= 'asc';
+
+        # https://metacpan.org/pod/SQL::Abstract#ORDER-BY-CLAUSES
         push @order_spec, { "-$dir" => $field };
     }
 
@@ -228,12 +301,13 @@ sub _handle_distinct_param {
     my @errors;
 
     # these restrictions avoid edge cases we don't want to deal with yet
-    push @errors, "distinct param requires order param"
-        unless $self->param('order');
+    my $sort = $self->param('sort') || $self->param('order'); # XXX insufficient
+    push @errors, "distinct param requires sort (or order) param"
+        unless $sort;
     push @errors, "distinct param requires fields param"
         unless $self->param('fields');
     push @errors, "distinct param requires fields and orders parameters to have same value"
-        unless $self->param('fields') eq $self->param('order');
+        unless $self->param('fields') eq $sort;
     my $errors = join(", ", @errors);
     die "$errors\n" if $errors; # TODO throw?
 
@@ -258,7 +332,7 @@ WebAPI::DBIC::Resource::Role::DBICParams
 
 =head1 VERSION
 
-version 0.001010
+version 0.003001
 
 =head1 NAME
 
