@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::Role::Router;
-$WebAPI::DBIC::Resource::Role::Router::VERSION = '0.001010'; # TRIAL
+$WebAPI::DBIC::Resource::Role::Router::VERSION = '0.003001'; # TRIAL
 
 use Moo::Role;
 
@@ -8,18 +8,21 @@ use Moo::Role;
 sub uri_for { ## no critic (RequireArgUnpacking)
     my $self = shift; # %pk in @_
 
-    my $env = $self->request->env;
-    my $router = $env->{'plack.router'};
-
-    my $url = $router->uri_for(@_)
+    my $url = $self->router->uri_for(@_)
         or return;
 
+    my $env = $self->request->env;
     my $prefix = $env->{SCRIPT_NAME};
     return "$prefix/$url" unless wantarray;
     return ($prefix, $url);
 }
 
 
+sub router {
+    return shift->request->env->{'plack.router'};
+}
+
+
 1;
 
 __END__
@@ -34,7 +37,7 @@ WebAPI::DBIC::Resource::Role::Router
 
 =head1 VERSION
 
-version 0.001010
+version 0.003001
 
 =head1 DESCRIPTION
 
